@@ -1,3 +1,6 @@
+"""This module provide methods for sampling large datasets for reducing the running time
+    
+"""
 import numpy as np
 import pandas as pd
 from typing import Union, List
@@ -9,15 +12,15 @@ from scipy.stats import ks_2samp
 from arfs.utils import is_list_of_str, is_list_of_bool, is_list_of_int
 
 
-def sample(df, n=1000, sample_weight=None, method='gower'):
-    """Sampling rows from a dataframe when random sampling is not 
+def sample(df, n=1000, sample_weight=None, method="gower"):
+    """Sampling rows from a dataframe when random sampling is not
     enough for reducing the number of rows.
     The strategies are either using hierarchical clustering
     based on the Gower distance or using isolation forest for identifying
     the most similar elements.
     For the clustering algorithm, clusters are determined using the Gower distance
     (mixed type data) and the dataset is shrunk from n_samples to n_clusters.
-    
+
     For the isolation forest algorithm, samples are added till a suffisant 2-samples
     KS statistics is reached or if the number iteration reached the max number (20)
 
@@ -36,51 +39,55 @@ def sample(df, n=1000, sample_weight=None, method='gower'):
     -------
     pd.DataFrame
         the sampled dataframe
-        
+
     """
     assert isinstance(df, pd.DataFrame), "X should be a DataFrame"
     X = df.copy()
     num_cols = list(X.select_dtypes(include=[np.number]))
     non_num_cols = list(set(list(X.columns)) - set(num_cols))
-    
-    if method == 'gower':
+
+    if method == "gower":
         # basic imputation
         if non_num_cols:
-            X[non_num_cols] =  X[non_num_cols].fillna(X[non_num_cols].mode().iloc[0])
-        if num_cols:    
-            X[num_cols] =  X[num_cols].fillna(X[num_cols].mean().iloc[0])
-        
+            X[non_num_cols] = X[non_num_cols].fillna(X[non_num_cols].mode().iloc[0])
+        if num_cols:
+            X[num_cols] = X[num_cols].fillna(X[num_cols].mean().iloc[0])
+
         # no need for scaling, it is built-in the computation of the Gower distance
         gd = gower_matrix(X, cat_features=non_num_cols, weight=sample_weight)
-        
-        labels = AgglomerativeClustering(n_clusters=n, affinity="precomputed", linkage="complete").fit_predict(gd)
-        X['label'] = labels
-        X['label'] = "clus_" + X['label'].astype(str)
-        X_nonnum = X.groupby('label')[non_num_cols].agg(get_most_common)
-        X_num = X.groupby('label')[num_cols].agg('mean')
+
+        labels = AgglomerativeClustering(
+            n_clusters=n, affinity="precomputed", linkage="complete"
+        ).fit_predict(gd)
+        X["label"] = labels
+        X["label"] = "clus_" + X["label"].astype(str)
+        X_nonnum = X.groupby("label")[non_num_cols].agg(get_most_common)
+        X_num = X.groupby("label")[num_cols].agg("mean")
         X_sampled = X_num.join(X_nonnum)
         X_sampled = X_sampled.reindex(X.columns, axis=1)
         return X_sampled
-    elif method == 'isoforest':
-        X[non_num_cols] = X[non_num_cols].astype('str').astype('category')
+    elif method == "isoforest":
+        X[non_num_cols] = X[non_num_cols].astype("str").astype("category")
         for col in non_num_cols:
-            X[col] = X[col].astype('category').cat.codes
+            X[col] = X[col].astype("category").cat.codes
         idx = isof_find_sample(X, sample_weight=None)
         return X.iloc[idx, :]
     else:
         NotImplementedError(f"{method} not implemented")
-        
-    
+
+
 def get_most_common(srs):
     x = list(srs)
     my_counter = Counter(x)
     return my_counter.most_common(1)[0][0]
 
 
-def gower_matrix(data_x: Union[np.array, pd.DataFrame],
-                 data_y: Union[np.array, pd.DataFrame, pd.Series] = None,
-                 weight: Union[np.array, pd.Series] = None,
-                 cat_features: Union[List[str], str, List[bool], List[int]] = 'auto'):
+def gower_matrix(
+    data_x: Union[np.array, pd.DataFrame],
+    data_y: Union[np.array, pd.DataFrame, pd.Series] = None,
+    weight: Union[np.array, pd.Series] = None,
+    cat_features: Union[List[str], str, List[bool], List[int]] = "auto",
+):
     """Computes the gower distances between X and Y
 
     Gower is a similarity measure for categorical, boolean and numerical mixed
@@ -101,7 +108,7 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
     -------
     np.array
         The Gower distance matrix, shape (n_samples, n_samples)
-    
+
     Notes
     -----
     The non-numeric features, and numeric feature ranges are determined from X and not Y.
@@ -136,7 +143,7 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
     x_n_rows, x_n_cols = X.shape
     y_n_rows, y_n_cols = Y.shape
 
-    if cat_features == 'auto':
+    if cat_features == "auto":
         if not isinstance(X, np.ndarray):
             is_number = np.vectorize(lambda x: not np.issubdtype(x, np.number))
             cat_features = is_number(X.dtypes)
@@ -153,10 +160,14 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
         elif is_list_of_bool(cat_features):
             cat_features = np.array(cat_features)
         elif is_list_of_int(cat_features):
-            cat_feat = [True if c in cat_features else False for c in range(len(X.columns))]
+            cat_feat = [
+                True if c in cat_features else False for c in range(len(X.columns))
+            ]
             cat_features = np.array(cat_feat)
         else:
-            raise TypeError("If not 'auto' cat_features should be a list of strings, integers or Booleans")
+            raise TypeError(
+                "If not 'auto' cat_features should be a list of strings, integers or Booleans"
+            )
 
     # print(cat_features)
 
@@ -189,10 +200,12 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
         num_ranges[col] = (1 - min_ / max_) if (max_ != 0) else 0.0
 
     # This is to normalize the numeric values between 0 and 1.
-    Z_num = np.divide(Z_num.astype(float),
-                      num_max.astype(float),
-                      out=np.zeros_like(Z_num).astype(float),
-                      where=num_max != 0)
+    Z_num = np.divide(
+        Z_num.astype(float),
+        num_max.astype(float),
+        out=np.zeros_like(Z_num).astype(float),
+        where=num_max != 0,
+    )
     Z_cat = Z[:, cat_features]
 
     if weight is None:
@@ -207,10 +220,18 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
 
     weight_sum = weight.sum()
 
-    X_cat = Z_cat[x_index, ]
-    X_num = Z_num[x_index, ]
-    Y_cat = Z_cat[y_index, ]
-    Y_num = Z_num[y_index, ]
+    X_cat = Z_cat[
+        x_index,
+    ]
+    X_num = Z_num[
+        x_index,
+    ]
+    Y_cat = Z_cat[
+        y_index,
+    ]
+    Y_num = Z_num[
+        y_index,
+    ]
 
     # print(X_cat,X_num,Y_cat,Y_num)
 
@@ -219,14 +240,16 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
         if x_n_rows != y_n_rows:
             j_start = 0
         # call the main function
-        res = _gower_distance_row(X_cat[i, :],
-                                  X_num[i, :],
-                                  Y_cat[j_start:y_n_rows, :],
-                                  Y_num[j_start:y_n_rows, :],
-                                  weight_cat,
-                                  weight_num,
-                                  weight_sum,
-                                  num_ranges)
+        res = _gower_distance_row(
+            X_cat[i, :],
+            X_num[i, :],
+            Y_cat[j_start:y_n_rows, :],
+            Y_num[j_start:y_n_rows, :],
+            weight_cat,
+            weight_num,
+            weight_sum,
+            num_ranges,
+        )
         # print(res)
         out[i, j_start:] = res
         if x_n_rows == y_n_rows:
@@ -235,14 +258,16 @@ def gower_matrix(data_x: Union[np.array, pd.DataFrame],
     return out
 
 
-def _gower_distance_row(xi_cat: np.array,
-                        xi_num: np.array,
-                        xj_cat: np.array,
-                        xj_num: np.array,
-                        feature_weight_cat: np.array,
-                        feature_weight_num: np.array,
-                        feature_weight_sum: np.array,
-                        ranges_of_numeric: np.array):
+def _gower_distance_row(
+    xi_cat: np.array,
+    xi_num: np.array,
+    xj_cat: np.array,
+    xj_num: np.array,
+    feature_weight_cat: np.array,
+    feature_weight_num: np.array,
+    feature_weight_sum: np.array,
+    ranges_of_numeric: np.array,
+):
     """Compute a row of the Gower matrix
 
     Parameters
@@ -275,7 +300,12 @@ def _gower_distance_row(xi_cat: np.array,
 
     # numerical columns
     abs_delta = np.absolute(xi_num - xj_num)
-    sij_num = np.divide(abs_delta, ranges_of_numeric, out=np.zeros_like(abs_delta), where=ranges_of_numeric != 0)
+    sij_num = np.divide(
+        abs_delta,
+        ranges_of_numeric,
+        out=np.zeros_like(abs_delta),
+        where=ranges_of_numeric != 0,
+    )
 
     sum_num = np.multiply(feature_weight_num, sij_num).sum(axis=1)
     sums = np.add(sum_cat, sum_num)
@@ -305,15 +335,17 @@ def smallest_indices(ary, n):
     indices = indices[np.argsort(flat[indices])]
     # indices = np.delete(indices,0,0)
     values = flat[indices]
-    return {'index': indices, 'values': values}
+    return {"index": indices, "values": values}
 
 
-def gower_topn(data_x: Union[np.array, pd.DataFrame],
-               data_y: Union[np.array, pd.DataFrame, pd.Series] = None,
-               weight: Union[np.array, pd.Series] = None,
-               cat_features: Union[List[str], str, List[bool], List[int]] = 'auto',
-               n: int = 5,
-               key: str = None):
+def gower_topn(
+    data_x: Union[np.array, pd.DataFrame],
+    data_y: Union[np.array, pd.DataFrame, pd.Series] = None,
+    weight: Union[np.array, pd.Series] = None,
+    cat_features: Union[List[str], str, List[bool], List[int]] = "auto",
+    n: int = 5,
+    key: str = None,
+):
     """Get the n most similar elements
 
     Parameters
@@ -329,8 +361,8 @@ def gower_topn(data_x: Union[np.array, pd.DataFrame],
     n : int, optional
         the number of neighbors/similar rows to find, by default 5
     key : str, optional
-        identifier key. If several rows refer to the same id, this column 
-        will be used for finding the nearest neighbors with a 
+        identifier key. If several rows refer to the same id, this column
+        will be used for finding the nearest neighbors with a
         different id, by default None
 
     Returns
@@ -343,7 +375,7 @@ def gower_topn(data_x: Union[np.array, pd.DataFrame],
     TypeError
         if the reference element is not a single row
     """
-    
+
     if data_y.shape[0] >= 2:
         raise TypeError("Only support `data_y` of 1 row. ")
     if key is None:
@@ -354,16 +386,16 @@ def gower_topn(data_x: Union[np.array, pd.DataFrame],
         dm = gower_matrix(Y, X, weight, cat_features)
 
     if key is not None:
-        idx = smallest_indices(np.nan_to_num(dm[0], nan=1), n)['index']
-        val = smallest_indices(np.nan_to_num(dm[0], nan=1), n)['values']
+        idx = smallest_indices(np.nan_to_num(dm[0], nan=1), n)["index"]
+        val = smallest_indices(np.nan_to_num(dm[0], nan=1), n)["values"]
         unique_id = data_x.iloc[idx, :]
         unique_id = unique_id[key]
         nunique_id = unique_id.nunique()
         mul = 1
         # continue looking for the closest n unique records with a different id
         while nunique_id < n:
-            idx = smallest_indices(np.nan_to_num(dm[0], nan=1), mul * n)['index']
-            val = smallest_indices(np.nan_to_num(dm[0], nan=1), mul * n)['values']
+            idx = smallest_indices(np.nan_to_num(dm[0], nan=1), mul * n)["index"]
+            val = smallest_indices(np.nan_to_num(dm[0], nan=1), mul * n)["values"]
             unique_id = data_x.iloc[idx, :].reset_index()
             unique_id = unique_id[key]
             nunique_id = unique_id.nunique()
@@ -377,7 +409,7 @@ def gower_topn(data_x: Union[np.array, pd.DataFrame],
         # sort them from the closest to the farthest, according to the Gower metrics
         idx_n = np.argsort(val)
         # return the n closest records, with a different id
-        return {'index': idx[idx_n[:n]], 'values': val[idx_n[:n]]}
+        return {"index": idx[idx_n[:n]], "values": val[idx_n[:n]]}
     else:
         return smallest_indices(np.nan_to_num(dm[0], nan=1), n)
 
@@ -388,16 +420,16 @@ def get_5_percent_splits(length):
     Parameters
     ----------
     length : int
-        array length 
+        array length
 
     Returns
     -------
     array
-        vector of sizes 
+        vector of sizes
     """
 
-    five_percent = round(5  / 100 * length)
-    return np.arange(five_percent, length, five_percent)    
+    five_percent = round(5 / 100 * length)
+    return np.arange(five_percent, length, five_percent)
 
 
 def isolation_forest(X, sample_weight=None):
@@ -410,13 +442,14 @@ def isolation_forest(X, sample_weight=None):
     sample_weight : pd.Series or np.array, optional
         the sample weights, if any, by default None
     """
-    clf = IsolationForest().fit(X, sample_weight = sample_weight)
+    clf = IsolationForest().fit(X, sample_weight=sample_weight)
     return clf.score_samples(X)
-    
+
+
 def isof_find_sample(X, sample_weight=None):
     """Finds a sample by comparing the distributions of the anomally scores between the sample and the original
     distribution using the KS-test. Starts of a 5% howver will increase to 10% and then 15% etc. if a significant sample can not be found
-    
+
     References
     ----------
     Sampling method taken from boruta_shap, author: https://github.com/Ekeany
@@ -439,11 +472,13 @@ def isof_find_sample(X, sample_weight=None):
     element = 1
     preds = isolation_forest(X, sample_weight)
     while loop:
-        sample_indices = np.random.choice(np.arange(preds.size),  size=size[element], replace=False)
+        sample_indices = np.random.choice(
+            np.arange(preds.size), size=size[element], replace=False
+        )
         sample = np.take(preds, sample_indices)
         if ks_2samp(preds, sample).pvalue > 0.95:
             break
         if iteration == 20:
-            element  += 1
+            element += 1
             iteration = 0
-    return sample_indices
\ No newline at end of file
+    return sample_indices
